"""This module is part of GMX_Simple_Analysis_Tool library. Written by CharlesHahn.

XPM module process xpm files, including get information from xpm, visualization and data convertion. 

This module requires Numpy, Scipy, Matplotlib and argparse. 

This module contains:
    class XPM
    function xpm_combine

This file is provided to you under GPLv2 License"""

## TODO: data convertion and process in XPM class

import os
import sys
import math
import argparse
import numpy as np
from scipy.interpolate import interp2d
from scipy.ndimage.filters import gaussian_filter
import matplotlib.pyplot as plt
from matplotlib.ticker import AutoLocator, FormatStrFormatter
from matplotlib import pylab as pylab

myparams = {
    "axes.labelsize": "12",
    "xtick.labelsize": "12",
    "ytick.labelsize": "12",
    "ytick.left": False,
    "ytick.direction": "in",
    "xtick.bottom": False,
    "xtick.direction": "in",
    "lines.linewidth": "2",
    "axes.linewidth": "1",
    "legend.fontsize": "12",
    "legend.loc": "upper right",
    "legend.fancybox": False,
    "legend.frameon": False,
    "font.family": "Arial",
    "font.size": 12,
    "figure.dpi": 150,
    "savefig.dpi": 300,
}
pylab.rcParams.update(myparams)


class XPM(object):
    """class XPM was defined to process xpm files

    Attributes:
    xpmfile: the name of xpm file
    xpm_title: the title of xpm file
    xpm_legend: the legend of xpm file
    xpm_type: two type of xpm file, "Continuous", "Discrete"
    xpm_xlabel: the x-label of xpm file
    xpm_ylabel: the y-label of xpm file
    xpm_width: the width of xpm file
    xpm_height: the height of xpm file
    xpm_color_num: the number of colors in xpm file
    xpm_char_per_pixel: the number of chars to represent a pixel of xpm figure
    chars: a list to store the chars which reprent pixels in xpm figure
    colors: a list to store the colors in xpm figure
    notes: a list to store the notes (comments to chars and colors) in xpm figure
    colors_rgb: a list of colors in rgb type
    xpm_xaxis: a list to store x-axis values
    xpm_yaxis: a list to store y-axis values
    xpm_datalines: a list to store each line of xpm figure content

    Methods:
    get_scatter_data(self): convert xpm figure content to scatter data
        return:
            scatter_x, scatter_y, x, y, v
    xpm2csv(self, outcsv: str = ""): convert xpm data to csv data
            outcsv: the output csv file name
    xpm2gpl(self, outgpl: str = ""): convert xpm data to gnuplot script
            outgpl: the output gpl file name
    draw_origin(self, IP: bool = False, outputpng: str = None, noshow: bool = False): draw xpm by plt.imshow()
            IP : whether to do interpolation
            outputpng : the output figure file name
            noshow: whether not to show figure, useful for computer without GUI
    draw_pcm(self, IP: bool = False, outputpng: str = None, noshow: bool = False): draw xpm by plt.pcolormesh()
            IP : whether to do interpolation
            outputpng : the output figure file name
            noshow: whether not to show figure, useful for computer without GUI
    draw_3D(self, IP: bool = False, outputpng: str = None, noshow: bool = False): draw xpm in 3D style
            IP : whether to do interpolation
            outputpng : the output figure file name
            noshow: whether not to show figure, useful for computer without GUI
    """

    def __init__(self, xpmfile):
        """read xpm file and save infos to class xpm"""

        self.xpmfile = xpmfile
        self.xpm_title = ""
        self.xpm_legend = ""
        self.xpm_type = ""
        self.xpm_xlabel = ""
        self.xpm_ylabel = ""
        self.xpm_width = 0
        self.xpm_height = 0
        self.xpm_color_num = 0
        self.xpm_char_per_pixel = 0
        self.chars = []
        self.colors = []
        self.notes = []
        self.colors_rgb = []
        self.xpm_xaxis = []
        self.xpm_yaxis = []
        self.xpm_datalines = []

        ## check kand read xpm file
        if not os.path.exists(xpmfile):
            print("ERROR -> no {} in current directory".format(xpmfile))
            exit()
        if xpmfile[-4:] != ".xpm":
            print("Error -> specify a xpm file with suffix xpm")
            exit()

        with open(xpmfile, "r") as fo:
            lines = [line.strip() for line in fo.readlines()]

        ## parse content of xpmfile
        flag_4_code = 0  ## means haven't detected yet
        for line in lines:
            ## find the 4 code line and parse
            if flag_4_code == 1:  ## means this line is code4 line
                flag_4_code = 2  ## means have detected
                code4 = [int(c) for c in line.strip().strip(",").strip('"').split()]
                self.xpm_width, self.xpm_height = code4[0], code4[1]
                self.xpm_color_num, self.xpm_char_per_pixel = code4[2], code4[3]
                continue
            elif (flag_4_code == 0) and line.startswith("static char"):
                flag_4_code = 1  ## means next line is code4 line
                continue

            ## parse comments and axis parts
            if line.startswith("/* x-axis"):
                self.xpm_xaxis += [float(n) for n in line.strip().split()[2:-1]]
                continue
            elif line.startswith("/* y-axis"):
                self.xpm_yaxis += [float(n) for n in line.strip().split()[2:-1]]
                continue
            elif line.startswith("/* title"):
                self.xpm_title = line.strip().split('"')[1]
                continue
            elif line.startswith("/* legend"):
                self.xpm_legend = line.strip().split('"')[1]
                continue
            elif line.startswith("/* x-label"):
                self.xpm_xlabel = line.strip().split('"')[1]
                continue
            elif line.startswith("/* y-label"):
                self.xpm_ylabel = line.strip().split('"')[1]
                continue
            elif line.startswith("/* type"):
                self.xpm_type = line.strip().split('"')[1]
                continue

            items = line.strip().split()
            ## for char-color-note part
            if len(items) == 7 and items[1] == "c":
                if len(items[0].strip('"')) == self.xpm_char_per_pixel:
                    self.chars.append(items[0].strip('"'))
                    self.colors.append(items[2])
                    self.notes.append(items[5].strip('"'))
                ## deal with blank
                if len(items[0].strip('"')) < self.xpm_char_per_pixel:
                    print("Warning -> space in char of line : {}".format(line))
                    char_item = items[0].strip('"')
                    self.chars.append(
                        char_item + " " * (self.xpm_char_per_pixel - len(char_item))
                    )
                    self.colors.append(items[2])
                    self.notes.append(items[5].strip('"'))
                continue

            ## for content part
            if line.strip().startswith('"') == 1 and (
                len(line.strip().strip(",").strip('"'))
                == self.xpm_width * self.xpm_char_per_pixel
            ):
                self.xpm_datalines.append(line.strip().strip(",").strip('"'))

        ## check infos
        if len(self.chars) != len(self.colors) != len(self.notes) != self.xpm_color_num:
            print("Wrong -> length of chars, colors, notes != xpm_color_num")
            print(
                "chars : {}, colors : {}, notes : {}, xpm_color_num : {}".format(
                    len(self.chars),
                    len(self.colors),
                    len(self.notes),
                    self.xpm_color_num,
                )
            )
            exit()

        if len(self.xpm_datalines) != self.xpm_height:
            print(
                "ERROR -> rows of data ({}) is not equal to xpm height ({}), check it !".format(
                    len(self.xpm_datalines), self.xpm_height
                )
            )
            exit()
        if (
            len(self.xpm_xaxis) != self.xpm_width
            and len(self.xpm_xaxis) != self.xpm_width + 1
        ):
            print(
                "ERROR -> length of x-axis ({}) != xpm width ({}) or xpm width +1".format(
                    len(self.xpm_xaxis), self.xpm_width
                )
            )
            exit()
        if (
            len(self.xpm_yaxis) != self.xpm_height
            and len(self.xpm_yaxis) != self.xpm_height + 1
        ):
            print(
                "ERROR -> length of y-axis ({}) != xpm height ({}) or xpm height +1".format(
                    len(self.xpm_yaxis), self.xpm_height
                )
            )
            exit()

        if len(self.xpm_xaxis) == self.xpm_width + 1:
            self.xpm_xaxis = [
                (self.xpm_xaxis[i - 1] + self.xpm_xaxis[i]) / 2.0
                for i in range(1, len(self.xpm_xaxis))
            ]
            print(
                "Warning -> length of x-axis is 1 more than xpm width, use intermediate value for instead. "
            )
        if len(self.xpm_yaxis) == self.xpm_height + 1:
            self.xpm_yaxis = [
                (self.xpm_yaxis[i - 1] + self.xpm_yaxis[i]) / 2.0
                for i in range(1, len(self.xpm_yaxis))
            ]
            print(
                "Warning -> length of y-axis is 1 more than xpm height, use intermediate value for instead. "
            )

        ## hex color to rgb values
        for color in self.colors:
            r = int(color[1:3], 16)
            g = int(color[3:5], 16)
            b = int(color[5:7], 16)
            self.colors_rgb.append([r, g, b])

        ## the read order of pixels is from top to bottom
        ## but the y-axis is from bottom to top, so reverse() is important !
        self.xpm_yaxis.reverse()

        print("Info -> all data has been read from {} successfully.".format(xpmfile))

    def get_scatter_data(self) -> tuple:
        """convert xpm data to scatter data"""

        ## parse xpm_data into x, y, v
        x, y, v = [], [], []
        scatter_x, scatter_y = [], []
        # print(len(xpm_xaxis))
        # print(len(xpm_yaxis))
        for l in range(len(self.xpm_datalines)):
            for i in range(
                0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
            ):
                v.append(
                    float(
                        self.notes[
                            self.chars.index(
                                self.xpm_datalines[l][i : i + self.xpm_char_per_pixel]
                            )
                        ]
                    )
                )
                x.append(self.xpm_xaxis[int(i / self.xpm_char_per_pixel)])
                y.append(self.xpm_yaxis[l])

        ## parse x, y, v into scatter_x, scatter_y
        v_max = max(v)
        scatter_weight = 1
        for i in range(len(v)):
            count = round((v_max - v[i]) * scatter_weight)
            for _ in range(count):
                scatter_x.append(x[i])
                scatter_y.append(y[i])

        return scatter_x, scatter_y, x, y, v

    def xpm2csv(self, outcsv: str = "") -> None:
        """convert xpm file to csv file
        outcsv: the name of output csv file
        """

        if outcsv == None:
            outcsv = ""
        if outcsv == "":
            outcsv = self.xpmfile[:-4] + ".csv"
        if outcsv[-4:] != ".csv":
            print("ERROR -> specify a output file with suffix csv")
            exit()
        if os.path.exists(outcsv):
            print("ERROR -> {} already in current directory".format(outcsv))
            exit()

        if self.xpm_type != "Continuous":
            print("ERROR -> can not extract data from xpm whose type is not Continuous")
            exit()

        ## only x, y, v values are needed
        _, _, x, y, v = self.get_scatter_data()
        if len(x) != len(y) != len(v):
            print("ERROR -> wrong in length of x, y, v")
            exit()
        ## write results
        with open(outcsv, "w") as fo:
            fo.write("{},{},{}\n".format("x-axis", "y-axis", "value"))
            for i in range(len(x)):
                fo.write("{:.6f},{:.6f},{:.6f}\n".format(x[i], y[i], v[i]))
        print("Info -> extract data from {} successfully".format(self.xpmfile))
        print("Info -> data are saved into {}".format(outcsv))

    def xpm2gpl(self, outgpl: str = "") -> None:
        """convert xpm file to gnuplot script
        outgpl: the name of output gnuplot script
        """

        ## check files
        if outgpl == None:
            outgpl = ""
        if outgpl == "":
            outgpl = self.xpmfile[:-4] + ".gpl"
        if os.path.exists(outgpl):
            print("ERROR -> {} already in current directory".format(outgpl))
            exit()
        outpng = self.xpmfile[:-4] + ".png"

        ## write gnuplot scripts
        gpl_lines = "set term png\n"
        gpl_lines += """set output "{}" \n""".format(outpng)
        gpl_lines += "unset colorbox\n"
        pal_line = "set pal defined("
        for index, color in enumerate(self.colors):
            pal_line += """{} "{}",""".format(index, color)
        pal_line = pal_line.strip(",") + ")"
        gpl_lines += pal_line + "\n\n"
        ## add data lines
        gpl_lines += "$data << EOD\n"
        for l in range(len(self.xpm_datalines)):
            for i in range(
                0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
            ):
                value = self.chars.index(
                    self.xpm_datalines[l][i : i + self.xpm_char_per_pixel]
                )
                gpl_lines += "{:.6f} {:.6f} {:.6f}\n".format(
                    self.xpm_xaxis[int(i / self.xpm_char_per_pixel)],
                    self.xpm_yaxis[l],
                    value,
                )
        gpl_lines += "EOD\n\n"
        ## add tail part of gpl file
        gpl_lines += "#set tmargin at screen 0.95\n"
        gpl_lines += "#set bmargin at screen 0.20\n"
        gpl_lines += "#set rmargin at screen 0.85\n"
        y_posi = 0.92
        for index, note in enumerate(self.notes):
            label_line = """#set label "{:10}" at screen 0.85,{:.2f} left textcolor rgb "{}"\n""".format(
                note, y_posi, self.colors[index]
            )
            y_posi -= 0.10
            gpl_lines += label_line
        gpl_lines += """set term pngcairo enhanced truecolor font "Arial,85" """
        gpl_lines += """fontscale 1 linewidth 20 pointscale 5 size 10000,6000\n"""
        gpl_lines += "set tics out nomirror;\n"
        gpl_lines += "set key out reverse Left spacing 2 samplen 1/2\n"
        gpl_lines += """set title "{}"\n""".format(self.xpm_title)
        gpl_lines += """set xlabel "{}"; set ylabel "{}";\n""".format(
            self.xpm_xlabel, self.xpm_ylabel
        )
        gpl_lines += """plot [{:.2f}:{:.2f}] [{:.2f}:{:.2f}] $data u 1:2:3 w imag notit, \\\n""".format(
            math.floor(min(self.xpm_xaxis) * 10.0) / 10.0 - 0.1,
            math.ceil(max(self.xpm_xaxis) * 10.0) / 10.0 + 0.1,
            math.floor(min(self.xpm_yaxis) * 10.0) / 10.0 - 0.1,
            math.ceil(max(self.xpm_yaxis) * 10.0) / 10.0 + 0.1,
        )
        for index, note in enumerate(self.notes):
            gpl_lines += """{} w p ps 3 pt 5 lc rgb "{}" t"{}", \\\n""".format(
                math.floor(min(self.xpm_yaxis)) - 1, self.colors[index], note
            )
        gpl_lines = gpl_lines.strip("\n").strip("\\").strip().strip(",")

        ## write gpl files
        with open(outgpl, "w") as fo:
            fo.write(gpl_lines + "\n")

        print(
            "Info -> write gnuplot scripts {} from {} successfully".format(
                outgpl, self.xpmfile
            )
        )

    def draw_origin(
        self, IP: bool = False, outputpng: str = None, noshow: bool = False
    ) -> None:
        """draw xpm figure by plt.imshow
        IP : whether to interpolation
        outputpng: the name for figure output
        noshow: whether not to show figure, useful for PC without gui
        """

        ## check parameters
        if outputpng != None and os.path.exists(outputpng):
            print("ERROR -> {} already in current directory".format(outputpng))
            exit()

        # visualization of xpm
        if IP == False:
            img = []
            for line in self.xpm_datalines:
                rgb_line = []
                for i in range(
                    0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
                ):
                    rgb_line.append(
                        self.colors_rgb[
                            self.chars.index(line[i : i + self.xpm_char_per_pixel])
                        ]
                    )
                img.append(rgb_line)

            plt.imshow(img, aspect="auto")

        if IP == True:
            if self.xpm_type != "Continuous":
                print("ERROR -> Only Continuous type xpm file can interpolation")
                exit()
            ## show figure with interpolation
            imgIP = []
            for line in self.xpm_datalines:
                value_line = []
                for i in range(
                    0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
                ):
                    value_line.append(
                        float(
                            self.notes[
                                self.chars.index(line[i : i + self.xpm_char_per_pixel])
                            ]
                        )
                    )
                imgIP.append(value_line)

            im = plt.imshow(imgIP, cmap="jet", interpolation="bilinear", aspect="auto")
            plt.colorbar(im, fraction=0.046, pad=0.04)

        ## TODO: find a better way to solve problem of ticks
        ## set the ticks
        x_tick, y_tick = 3, 3
        xpm_xticks = ["{:.1f}".format(x) for x in self.xpm_xaxis]
        xpm_yticks = ["{:.1f}".format(y) for y in self.xpm_yaxis]
        if self.xpm_width < 100:
            x_tick = int(self.xpm_width / 3)
        elif self.xpm_width >= 100 and self.xpm_width < 1000:
            x_tick = int(self.xpm_width / 5)
        elif self.xpm_width > 500:
            x_tick = int(self.xpm_width / 10)
        if self.xpm_height < 100:
            y_tick = int(self.xpm_height / 3)
        elif self.xpm_height >= 100 and self.xpm_height < 1000:
            y_tick = int(self.xpm_height / 5)
        elif self.xpm_height > 500:
            y_tick = int(self.xpm_height / 10)
        if self.xpm_width / self.xpm_height > 10:
            y_tick = int(self.xpm_height / 2)
        if self.xpm_height / self.xpm_width > 10:
            x_tick = int(self.xpm_width / 2)
        plt.tick_params(axis="both", which="major")
        plt.xticks(
            [0]
            + [w for w in range(x_tick, self.xpm_width - int(x_tick / 2), x_tick)]
            + [self.xpm_width - 1],
            [xpm_xticks[0]]
            + [
                xpm_xticks[w]
                for w in range(x_tick, self.xpm_width - int(x_tick / 2), x_tick)
            ]
            + [xpm_xticks[-1]],
        )
        plt.yticks(
            [0]
            + [h for h in range(y_tick, self.xpm_height - int(y_tick / 2), y_tick)]
            + [self.xpm_height - 1],
            [xpm_yticks[0]]
            + [
                xpm_yticks[h]
                for h in range(y_tick, self.xpm_height - int(y_tick / 2), y_tick)
            ]
            + [xpm_yticks[-1]],
        )

        ## set other infos in the figure
        plt.title(self.xpm_title)
        plt.xlabel(self.xpm_xlabel)
        plt.ylabel(self.xpm_ylabel)
        print("Legend of this xpm figure -> ", self.xpm_legend)

        if outputpng != None:
            plt.savefig(outputpng, dpi=300)
        if noshow == False:
            plt.show()

    def draw_pcm(
        self, IP: bool = False, outputpng: str = None, noshow: bool = False
    ) -> None:
        """draw xpm figure by pcolormesh (with interpolation)
        IP : whether to interpolation
        outputpng: the name for figure output
        noshow: whether not to show figure, useful for PC without gui
        """

        ## check parameters
        if outputpng != None and os.path.exists(outputpng):
            print("ERROR -> {} already in current directory".format(outputpng))
            exit()

        if self.xpm_type != "Continuous":
            print("ERROR -> Only Continuous type xpm file can interpolation")
            exit()

        ## convert xpm_data to img (values)
        img = []
        for line in self.xpm_datalines:
            value_line = []
            for i in range(
                0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
            ):
                value_line.append(
                    float(
                        self.notes[
                            self.chars.index(line[i : i + self.xpm_char_per_pixel])
                        ]
                    )
                )
            img.append(value_line)

        if IP == False:
            plt.pcolormesh(
                self.xpm_xaxis, self.xpm_yaxis, img, cmap="jet", shading="auto"
            )
        elif IP == True:
            ## interpolation
            ip_func = interp2d(self.xpm_xaxis, self.xpm_yaxis, img, kind="linear")
            x_new = np.linspace(
                np.min(self.xpm_xaxis), np.max(self.xpm_xaxis), 10 * len(self.xpm_xaxis)
            )
            y_new = np.linspace(
                np.min(self.xpm_yaxis), np.max(self.xpm_yaxis), 10 * len(self.xpm_yaxis)
            )
            value_new = ip_func(x_new, y_new)
            x_new, y_new = np.meshgrid(x_new, y_new)
            ## show figure
            plt.pcolormesh(x_new, y_new, value_new, cmap="jet", shading="auto")

        ## set ticks and other figure infos
        ax = plt.gca()
        ax.yaxis.set_major_formatter(FormatStrFormatter("%.1f"))
        ax.xaxis.set_major_formatter(FormatStrFormatter("%.1f"))
        plt.colorbar()
        plt.title(self.xpm_title)
        plt.xlabel(self.xpm_xlabel)
        plt.ylabel(self.xpm_ylabel)
        print("Legend of this xpm figure -> ", self.xpm_legend)

        if outputpng != None:
            plt.savefig(outputpng, dpi=300)
        if noshow == False:
            plt.show()

    def draw_3D(
        self, IP: bool = False, outputpng: str = None, noshow: bool = False
    ) -> None:
        """draw xpm 3D figure (with interpolation)
        IP : whether to interpolation
        outputpng: the name for figure output
        noshow: whether not to show figure, useful for PC without gui
        """

        ## check parameters
        if outputpng != None and os.path.exists(outputpng):
            print("ERROR -> {} already in current directory".format(outputpng))
            exit()

        if self.xpm_type != "Continuous":
            print("ERROR -> Only Continuous type xpm file can draw 3D figure")
            exit()

        ## convert xpm_data to values
        values = []
        for line in self.xpm_datalines:
            for i in range(
                0, self.xpm_width * self.xpm_char_per_pixel, self.xpm_char_per_pixel
            ):
                values.append(
                    float(
                        self.notes[
                            self.chars.index(line[i : i + self.xpm_char_per_pixel])
                        ]
                    )
                )
        xpm_xaxis = np.array(self.xpm_xaxis)
        xpm_yaxis = np.array(self.xpm_yaxis)
        img = np.array(values)

        ## draw 3d figure
        fig = plt.figure()
        ax = fig.gca(projection="3d")

        ## interpolation
        IP_value = 1
        if IP == False:
            IP_value = 1
        elif IP == True:
            IP_value = 12
        img = img.reshape(len(xpm_xaxis), len(xpm_yaxis))
        ip_func = interp2d(xpm_xaxis, xpm_yaxis, img, kind="linear")
        x_new = np.linspace(
            np.min(xpm_xaxis), np.max(xpm_xaxis), IP_value * len(xpm_xaxis)
        )
        y_new = np.linspace(
            np.min(xpm_yaxis), np.max(xpm_yaxis), IP_value * len(xpm_yaxis)
        )
        img_new = ip_func(x_new, y_new)
        x_new, y_new = np.meshgrid(x_new, y_new)
        img_new = img_new.reshape(len(x_new), len(y_new))

        ## show figure
        surf = ax.plot_surface(
            x_new,
            y_new,
            img_new,
            alpha=0.9,
            cmap="coolwarm",
            linewidth=0,
            antialiased=False,
        )
        ## set the 2d surface location
        ax.contourf(
            x_new,
            y_new,
            img_new,
            zdir="z",
            offset=math.floor(min(values)) - math.floor(max(values) - min(values)) / 30,
            cmap="coolwarm",
        )

        ## set the axis ticks and other figure infos
        ax.zaxis.set_major_locator(AutoLocator())
        ax.zaxis.set_major_formatter(FormatStrFormatter("%.1f"))
        ax.yaxis.set_major_formatter(FormatStrFormatter("%.1f"))
        ax.xaxis.set_major_formatter(FormatStrFormatter("%.1f"))
        # ax.set_zlim(0, )
        plt.colorbar(surf, shrink=0.6, aspect=12)
        plt.title(self.xpm_title)
        plt.xlabel(self.xpm_xlabel)
        plt.ylabel(self.xpm_ylabel)
        print("Legend of this xpm figure -> ", self.xpm_legend)

        if outputpng != None:
            plt.savefig(outputpng, dpi=300)
        if noshow == False:
            plt.show()


def xpm_combine(
    xpm_files: list = [], outputpng: str = None, noshow: bool = False
) -> None:
    """Combination of xpm files, still in construction, not suggest to use it
    xpm_file_list : a list contains all xpm file names
    outputpng : the name for figure output
    noshow: whether not to show figure, useful for PC without gui
    """

    ###################### bolzmann
    #### G(x) = -kT*Ln(P(x)) + c
    ######################

    x_list, y_list = [], []
    xpm_title, xpm_legend, xpm_xlabel, xpm_ylabel = "", "", "", ""
    for file in xpm_files:
        xpm = XPM(file)
        xpm_title = xpm.xpm_title
        xpm_legend = xpm.xpm_legend
        xpm_xlabel = xpm.xpm_xlabel
        xpm_ylabel = xpm.xpm_ylabel
        if xpm.xpm_type != "Continuous":
            print("ERROR -> can not combine xpm whose type is not Continuous")
            exit()
        scatter_x, scatter_y, _, _, _ = xpm.get_scatter_data()
        x_list += scatter_x
        y_list += scatter_y

    ## combine xpm
    # plt.scatter(x_list, y_list)
    # plt.show()
    heatmap, xedges, yedges = np.histogram2d(x_list, y_list, bins=800)
    heatmap = gaussian_filter(heatmap, sigma=16)
    extent = [xedges[0], xedges[-1], yedges[0], yedges[-1]]
    plt.imshow(heatmap.T, origin="lower", extent=extent, cmap="jet_r")
    plt.xlim(extent[0], extent[1])
    plt.ylim(extent[2], extent[3])

    ## set ticks and other figure infos
    ax = plt.gca()
    ax.yaxis.set_major_formatter(FormatStrFormatter("%.1f"))
    ax.xaxis.set_major_formatter(FormatStrFormatter("%.1f"))
    plt.title(xpm_title)
    plt.xlabel(xpm_xlabel)
    plt.ylabel(xpm_ylabel)
    print("Legend of this xpm figure -> ", xpm_legend)

    if outputpng != None and os.path.exists(outputpng):
        print("ERROR -> {} already in current directory".format(outputpng))
        exit()
    if outputpng != None:
        plt.savefig(outputpng, dpi=300)
    if noshow == False:
        plt.show()


def xpm_call_functions(arguments: list = None):
    """call functions of xpm module"""
    if arguments == None:
        arguments = [argv for argv in sys.argv]

    ## parse the command parameters
    parser = argparse.ArgumentParser(description="Process xpm files generated by GMX")
    parser.add_argument("-f", "--input", help="input your xpm file")
    parser.add_argument("-o", "--output", help="file name to output")
    parser.add_argument(
        "-ip",
        "--interpolation",
        action="store_true",
        help="whether to apply interpolation (only support Continuous type xpm)",
    )
    parser.add_argument(
        "-pcm",
        "--pcolormesh",
        action="store_true",
        help="whether to apply pcolormesh function to draw",
    )
    parser.add_argument(
        "-3d",
        "--threeDimensions",
        action="store_true",
        help="whether to draw 3D figure",
    )
    parser.add_argument(
        "-ns",
        "--noshow",
        action="store_true",
        help="whether not to show picture, useful on computer without gui",
    )
    parser.add_argument(
        "-c",
        "--combine",
        nargs="+",
        help="specify some xpm files to combine into one figure",
    )

    if len(arguments) < 2:
        print("Error -> no input parameters, -h or --help for help messages")
        exit()

    method = arguments[1]
    # print(method)
    if method in ["-h", "--help"]:
        parser.parse_args(arguments[1:])
        exit()
    args = parser.parse_args(arguments[2:])
    # for key, value in vars(args).items():
    #     print(key, value)

    inputxpm = args.input
    output = args.output
    ip = args.interpolation
    noshow = args.noshow
    pcm = args.pcolormesh
    fig_3d = args.threeDimensions
    xpms2combine = args.combine

    ## call functions
    if method == "xpm_show":
        if inputxpm == None:
            print("Error -> no input file")
            exit()
        xpm = XPM(inputxpm)
        if fig_3d == True:
            xpm.draw_3D(ip, output, noshow)
        if pcm == False and fig_3d == False:
            xpm.draw_origin(ip, output, noshow)
        elif pcm == True and fig_3d == False:
            xpm.draw_pcm(ip, output, noshow)
    elif method == "xpm_combine":
        if xpms2combine == None:
            print("Error -> no input xpm files for combination")
            exit()
        xpm_combine(xpms2combine, output, noshow)
    elif method == "xpm2csv":
        xpm = XPM(inputxpm)
        xpm.xpm2csv(output)
    elif method == "xpm2gpl":
        xpm = XPM(inputxpm)
        xpm.xpm2gpl(output)
    else:
        print("Error -> Wrong method you specified")
        exit()

    print("Good Day !")


def main():
    xpm_call_functions()


if __name__ == "__main__":
    main()



""" HELP module is part of DuIvyTools library, which is a tool for analysis and visualization of GROMACS result files. This module is written by CharlesHahn.

This module is designed to provide help messages. 

This HELP module contains:
    - HELP class

This file is provided to you by GPLv2 license."""


import sys


class HELP(object):
    """HELP class provides help infos about all method in DuIvyTools"""

    def __init__(self) -> None:
        """init all help messages"""

        self.help_infos = {
            "xvg_show": """
xvg_show: draw xvg data into line figures.

:examples: 
    dit xvg_show -f test.xvg

:parameters:
    -f, --input
            specify the xvg file for input
""",
            "xvg_compare": """
xvg_compare: comparison of xvg files, draw different data columns into line figure.

:examples:
    dit xvg_compare -f f1.xvg f2.xvg -c 1,2 2,3,4 -l l1 l2 l3 l4 l5
    dit xvg_compare -f f1.xvg f2.xvg -c 1 1 -l l1 l2 -x Time(ns) -y ylabel -t tile
    dit xvg_compare -f f1.xvg f2.xvg -c 1,2 2 -l l1 l2 l3 -s 100 -e 1000

:parameters:
    -f, --input
            specify the xvg files for input. accept one or one more input file names 
            seperated by space
    -c, --column_select
            specify the index of columns you want to use. index starts from 0. 
            one xvg file in -f should be paired with one column index group seperated by
            space, and in each column index group, indexs should be seperated by comma. 
            In first example, 1,2 is paired with f1.xvg, 2,3,4 is paird with f2.xvg; 
            this means selecting the column 1 and 2 in f1.xvg, column 2, 3 and 4 in 
            f2.xvg to compare. 
    -l, --legend_list (optional)
            specify the legends to show in figure. The number of legends should be equal
            to the number of indexs you sepecified in -c. 
            if you want space in one legend, like "hh gg"; use "~~" for instead, like
            "hh~~gg", ~~ will be turn into space and shown in figure.
    -s, --start (optional)
            specify the row index of column data which you want to start to plot.
    -e, --end (optional)
            specify the row index of column data which you want to stop plotting.
    -x, --xlabel (optional)
            sepecify the x-label you want to show in figure
    -y, --ylabel (optional)
            sepecify the y-label you want to show in figure
    -t, --title (optional)
            sepecify the title you want to show in figure
    -smv, --showMV  (optional)
            if you specified this parameter, moving average and confidence interval 
            will be calculated and shown in figure. 
    -ws, --windowsize (optional)
            specify the windows size for moving average calculation. default == 50
            only valid when -smv specified. 
    -cf, --confidence (optional)
            specify the confidence for confidence interval calculation. default == 0.90
            only valid when -smv specified. 
    -a, --alpha (optional)
            specify the transparency for plotting confidence interval. default == 0.4
            only valid when -smv specified. 
""",
            "xvg_ave": """
xvg_ave: calculate the average of each column in xvg file

:examples:
    dit xvg_ave -f f1.xvg
    dit xvg_ave -f f1.xvg -s 1000 -e 2000

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -s, --start (optional)
            specify the row index of column data which you want to start to calculation.
    -e, --end (optional)
            specify the row index of column data which you want to stop calculation.
""",
            "xvg_mvave": """
xvg_mvave: calculate the moving average of each column in xvg file and save results to csv file.

:examples:
    dit xvg_mvave -f f1.xvg -o res.csv 
    dit xvg_mvave -f f1.xvg -o res.csv -ws 20 -cf 0.95

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -o, --output
            specify the csv file to save results.
    -ws, --windowsize (optional)
            specify the windows size for moving average calculation. default == 50
    -cf, --confidence (optional)
            specify the confidence for confidence interval calculation. default == 0.90
""",
            "xvg2csv": """
xvg2csv: convert xvg file into csv file.

:examples:
    dit xvg2csv -f f1.xvg -o res.csv

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -o, --output
            specify the csv file to save results.
""",
            "xvg_rama": """
xvg_rama: draw ramachandran figure from a rama data xvg file.

:examples:
    dit xvg_rama -f rama.xvg

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
""",
            "xvg_show_distribution": """
xvg_show_distribution: calculate distribution of each column in xvg file and show.

:examples:
    dit xvg_show_distribution -f f1.xvg
    dit xvg_show_distribution -f f1.xvg -bin 100

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -bin, --bin (optional)
            specify the number of bins of calculating distribution. default == 100
""",
            "xvg_show_stack": """
xvg_show_stack: draw stack figure of column data.

:examples:
    dit xvg_show_stack -f f1.xvg
    dit xvg_show_stack -f f1.xvg -c 2 3 4 5 6 -s 500 -e 1000
    dit xvg_show_stack -f f1.xvg -c 2 3 4 5 6 -l A B C D E

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -c, --column_select
            specify the column indexs you want to stack,  seperated by space.
    -l, --legend_list (optional)
            Normally, legends could be assigned automatically, but you can specify them
            by -l. The number of legends should be equal to the number of column indexs.
            if you want space in one legend, like "hh gg"; use "~~" for instead, like
            "hh~~gg", ~~ will be turn into space and shown in figure.
    -s, --start (optional)
            specify the row index of column data which you want to start to calculation.
    -e, --end (optional)
            specify the row index of column data which you want to stop calculation.
""",
            "xvg_show_scatter": """
xvg_show_scatter: draw scatter figure

:examples:
    dit xvg_show_scatter -f f1.xvg -xi 0 -yi 1

:parameters:
    -f, --input
            specify the xvg file for input. 
            Only the first file is valid, others will be dropped.
    -xi, --x_index (optional)
            specify the index of column which you want to use as x data. default == 0
    -yi, --y_index (optional)
            specify the index of column which you want to use as y data. default == 1
""",
            "xvg_energy_compute": """
xvg_energy_compute: calculate the interaction energy between two items.

:examples:
    dit xvg_energy_compute -f prolig.xvg pro.xvg lig.xvg -o energy_results.xvg

:parameters:
    -f, --input
            specify three xvg files to calculate (complex energy file, component A 
            energy file and component B energy file). Each should contain and ONLY 
            contain five columns (Time, LJ(SR), Disper.corr., Coulomb(SR), Coul.recip.)
    -o, --output
            specify a xvg file to save results. 
""",
            "xvg_combine": """
xvg_combine: select columns from xvg files and conbine data columns into one xvg file. 

:examples:
    dit xvg_combine -f f1.xvg -c 1,2 -o res.xvg
    dit xvg_combine -f f1.xvg f2.xvg -c 1,2 2,3 -o res.xvg

:parameters:
    -f, --input
            specify the xvg files for input. accept one or one more input file names 
            seperated by space
    -c, --column_select
            specify the index of columns you want to use. index starts from 0. 
            one xvg file in -f should be paired with one column index group seperated by
            space. And in each column index group, indexs should be seperated by comma. 
            In second example, 1,2 is paired with f1.xvg, 2,3 is paird with f2.xvg; 
            this means selecting the column 1 and 2 in f1.xvg, column 2 and 2 in 
            f2.xvg to combine, and saving results to res.xvg. 
    -o, --output
            specify a xvg file to save results of combination.
""",
            "xvg_ave_bar": """
xvg_ave_bar: First, the average of each data column you select by -c will be calculated.
             Second, the average and standard error of corresponding column averages in 
             xvg file groups will be calculated.
             Third, the averages and standard errors will be draw into bar figure.

:examples:
    dit xvg_ave_bar -f f1_1.xvg,f1_2.xvg,f1_3.xvg f2_1.xvg,f2_2.xvg,f2_3.xvg -c 1 2 
    dit xvg_ave_bar -f f1.xvg f2.xvg -c 1 2 -l l1 l2 -s 100 -e 1000 
    dit xvg_ave_bar -f f1.xvg f2.xvg -c 1 2 -l l1 l2 -x xlabel -y ylabel -t title
    dit xvg_ave_bar -f f1.xvg f2.xvg f3.xvg -c 1 2 -l l1 l2 l3 -xt xt1 xt2
    dit xvg_ave_bar -f f1.xvg f2.xvg -c 1 2 -l l1 l2 -ac -o average.csv

:parameters:
    -f, --input
            specify the xvg files for input. accept one or one more file names or file 
            name groups. Several file names seperated by comma is called a file name 
            group. In first example, f1_1.xvg,f1_2.xvg,f1_3.xvg is the first file name 
            group, and f2_1.xvg,f2_2.xvg,f2_3.xvg is the second file name group. 
            This method will calculate averages of columns you select in each xvg file.
            Then calculate the average and standard error of column averages in each 
            file group. Finally, convert the averages and standard errors into bar plot.
    -c, --column_select
            specify the index of columns you want to calculate. Index starts from 0. 
            The column indexs you select will be applied to EVERY xvg file in -f.
    -l, --legend_list (optional)
            specify the legends to show in figure. The number of legends should be equal
            to the number of files or file groups you specified in -f. 
            if you want space in one legend, like "hh gg"; use "~~" for instead, like
            "hh~~gg", ~~ will be turn into space and shown in figure.
    -xt, --xtitles (optional)
            specity the names of x ticks in figure. The number of xtitles should be 
            equal to the number of indexs in column_select (-c).
    -s, --start (optional)
            specify the row index of column data which you want to start to calculation.
    -e, --end (optional)
            specify the row index of column data which you want to stop calculation.
    -x, --xlabel (optional)
            sepecify the x-label you want to show in figure
    -y, --ylabel (optional)
            sepecify the y-label you want to show in figure
    -t, --title (optional)
            sepecify the title you want to show in figure
    -ac, --ave2csv (optional)
            whether save data of averages into csv file. default == False
    -o, --output (optional)
            if you specified -ac, -o should be used to specify the csv file name.
""",
            "xvg_box": """
xvg_box: draw the data of columns you select into box figure.

:examples:
    dit xvg_box -f f1.xvg f2.xvg f3.xvg -c 1 2 -xt xt1 xt2 -s 100 -e 1000
    dit xvg_box -f f1.xvg f2.xvg -c 1 2 3 -xt xt1 xt2 xt3 -x xlabel -y ylabel -t title

:parameters:
    -f, --input
            accept one or more than one files as input.
    -c, --column_select
            specify the column indexs you want to plot. The indexs you specified will
            be applied to each file you select in -f.
    -xt, --xtitles (optional)
            specity the names of x ticks in figure. The number of xtitles should be 
            equal to the number of indexs in column_select (-c).
    -s, --start (optional)
            specify the row index of column data which you want to start to calculation.
    -e, --end (optional)
            specify the row index of column data which you want to stop calculation.
    -x, --xlabel (optional)
            sepecify the x-label you want to show in figure
    -y, --ylabel (optional)
            sepecify the y-label you want to show in figure
    -t, --title (optional)
            sepecify the title you want to show in figure
""",
            "xpm_show": """
xpm_show: vasualize the xpm file.

:examples:
    dit xpm_show -f f1.xpm 
    dit xpm_show -f f1.xpm -ip 
    dit xpm_show -f f1.xpm -pcm 
    dit xpm_show -f f1.xpm -3d
    dit xpm_show -f f1.xpm -o test.png -ns 

:parameters:
    -f, --input
            specify the xpm file for input. 
    -o, --output (optional)
            specify the output figure name. 
    -ip, --interpolation (optional)
            whether to perform interpolation which will make your figure looks better.
    -pcm, --pcolormesh (optional)
            whether to use pcolormesh function to vasualize your xpm figure.
    -3d, --threeDimensions (optional)
            whether to draw a 3D figure. Useful for free energy landscape figure.
    -ns, --noshow (optional)
            whether not to show figure in GUI, useful when working without GUI.
""",
            "xpm2csv": """
xpm2csv: convert xpm file into csv data file. 
         when number of x-axis (or y-axis) is 1 more than width (or height) of figure, 
         the intermediate value of two adjacent values of x-axis (or y-axis) will be 
         calculated to suit the width (or height).

:examples:
    dit xpm2csv -f f1.xpm -o test.csv

:parameters:
    -f, --input
            specify the xpm file for input. 
    -o, --output
            specify the output csv file name.
""",
            "xpm2gpl": """
xpm2csv: convert xpm file into gnuplot script.

:examples:
    dit xpm2csv -f f1.xpm -o test.gpl

:parameters:
    -f, --input
            specify the xpm file for input. 
    -o, --output
            specify the output gnuplot script file name.
""",
            "xpm_combine": """
xpm_combine: combine several xpm files into one figure. 
             STILL WORKING ON IT. NOT RECOMMAND TO USE IT !!!

:examples:
    dit xpm_combine -c f1.xpm f2.xpm
    dit xpm_combine -c f1.xpm f2.xpm -o test.png -ns

:parameters:
    -c, --combine
            specify several xpm files for input. 
    -o, --output
            specify the output figure name. 
    -ns, --noshow (optional)
            whether not to show figure in GUI, useful when working without GUI.
""",
            "ndx_show": """
ndx_show: print all group names in ndx file you specified.

:examples:
    dit ndx_show -f f1.ndx

:parameters:
    -f, --input
            specify the ndx file for input. 
""",
            "ndx_rm_dup": """
ndx_rm_dup: remove all duplicated groups in ndx file.

:examples:
    dit ndx_rm_dup -f f1.ndx -o res.ndx

:parameters:
    -f, --input
            specify the ndx file for input. 
    -o, --output
            specify the ndx file name for saving the results of romiving.
""",
            "ndx_rm": """
ndx_rm: remove index groups according to names you specified.

:examples:
    dit ndx_rm -f f1.ndx -o res.ndx -gl Protein SOL System
    dit ndx_rm -f f1.ndx -o res.ndx -int

:parameters:
    -f, --input
            specify the ndx file for input. 
    -o, --output
            specify the ndx file name for saving the results.
    -gl, --grouplist (optional)
            specify the group names you want to remove. Seperated by space.
    -int, --interactive (optional)
            specify the group names you want to remove by interactive mode.
""",
            "ndx_preserve": """
ndx_preserve: preserve index groups according to names you specified, and remove others.

:examples:
    dit ndx_preserve -f f1.ndx -o res.ndx -gl Protein SOL System
    dit ndx_preserve -f f1.ndx -o res.ndx -int

:parameters:
    -f, --input
            specify the ndx file for input. 
    -o, --output
            specify the ndx file name for saving the results.
    -gl, --grouplist (optional)
            specify the group names you want to preserve. Seperated by space.
    -int, --interactive (optional)
            specify the group names you want to preserve by interactive mode.
""",
            "ndx_add": """
ndx_add: add one index group to ndx file by specified group name, start, end, step.

:examples:
    dit ndx_add -f f1.ndx -o res.ndx -gn test -s 1 -e 10 -t 2

:parameters:
    -f, --input (optional)
            specify ndx file for input. -f could be ignored if you want a new ndx file.
    -o, --output
            specify the ndx file name for saving the results.
    -gn, --groupname
            specify the name of index group you want to add.
    -s, --start
            specify the start index of the group you want to add.
    -e, --end
            specify the end index of the group you want to add.
    -t, --step (optional)
            specify the step of indexs of the group you want to add. default == 1
            In the example, -s 1 -e 10 -t 2 means to generate a group contains index:
            1 3 5 7 9 
""",
            "ndx_combine": """
ndx_combine: combine groups into one group by the group names you specified.
             I recommand you to use gmx make_ndx command to do this.

:example:
    dit ndx_combine -f f1.ndx -o res.ndx -gn combined_group -gl Protein Ligand

:parameters:
    -f, --input
            specify the ndx file for input. 
    -o, --output
            specify the ndx file name for saving the results.
    -gn, --groupname
            specify the name of combined index group.
    -gl, --grouplist
            specify the group names you want to combine. Seperated by space.
""",
            "ndx_rename": """
ndx_rename: rename groups in ndx file.

:examples:
    dit ndx_rename -f f1.ndx -o res.ndx -on Protein -nn test
    dit ndx_rename -f f1.ndx -o res.ndx -int

:parameters:
    -f, --input
            specify the ndx file for input. 
    -o, --output
            specify the ndx file name for saving the results.
    -on, --oldname (optional)
            specify the old name of group you want to rename.
    -nn, --newname (optional)
            specify a new name for the group you renamed.
    -int, --interactive (optional)
            rename groups in interactive mode.
""",
            "mdp_gen": """
mdp_gen: generate a template mdp file by application you specified. The mdp file 
         generated may NOT be appropriate for your system, CHECK IT YOURSELF.

:examples:
    dit mdp_gen -o em.mdp -a em
    dit mdp_gen -o npt.mdp -a npt

:parameters:
    -o, --output
            specify an output mdp file to save the results.
    -a, --application
            specify the applications you want to use mdp file for. You can select from:
            ions, em, nvt, npt, md, blank
""",
        }

    def print_help_infos(self, method: str) -> None:
        """print help messages"""

        if method in self.help_infos.keys():
            print(self.help_infos[method])
        else:
            print(
                "Error -> unknown method {}, type `dit help` for more infos.".format(
                    method
                )
            )
            exit()


def print_help_msg(method: str) -> None:
    """print help messages"""

    help_class = HELP()
    help_class.print_help_infos(method)


def help_call_functions(arguments: list = None) -> None:
    """call functions according to arguments"""

    if arguments == None:
        arguments = [argv for argv in sys.argv]

    description = "HELP method provides help infos about all methods in DuIvyTools\n"
    description += "Type any methods below to show help messages:\n"
    description += "like: `dit help xvg_show` or `dit xvg_show -h`\n"
    description += "    XVG:\n"
    description += "        xvg_show, xvg_compare, xvg_ave, xvg_mvave, "
    description += "xvg2csv, xvg_rama\n"
    description += "        xvg_show_distribution, xvg_show_stack, xvg_show_scatter\n"
    description += "        xvg_energy_compute, xvg_combine, xvg_ave_bar, xvg_box\n"
    description += "    XPM:\n"
    description += "        xpm_show, xpm2csv, xpm2gpl, xpm_combine\n"
    description += "    NDX:\n"
    description += "        ndx_show, ndx_rm_dup, ndx_rm, ndx_preserve\n"
    description += "        ndx_add, ndx_combine, ndx_rename\n"
    description += "    MDP:\n"
    description += "        mdp_gen\n"

    if len(arguments) < 2:
        print("DuIvyTools is a simple analysis and visualization tool ", end="")
        print("for GROMACS result files.")
        print("Info -> type `dit help` for more messages")
        exit()
    elif len(arguments) == 2:
        if arguments[1] in ["help", "-h", "--help"]:
            print(description)
        else:
            print("Error -> unknown command, type `dit help` for more information")
            exit()
    else:
        if arguments[1] != "help":
            print("Error -> unknown command {}".format(arguments[1]))
            exit()
        else:
            methods = arguments[2:]
            for method in methods:
                print_help_msg(method)


def main():
    help_call_functions()


if __name__ == "__main__":
    main()



"""``DuIvyTools`` package for analysis of GMX MD results files. Written by Charles Hahn.

Python modules
----------------
The package consists of the following Python modules:
* DIT : the CLI application.
* XPM : module to process XPM files.
* XVG : module to process XVG files.
* NDX : module to process NDX files.
* MDP : module to generate MDP files.
* HELP : module to provide help messages.

"""




""" MDP module is part of DuIvyTools library, which is a tool for analysis and visualization of GROMACS result files. This module is written by CharlesHahn.

This module is designed to generate .mdp file templates. 

This MDP module contains:
    - MDP class

This file is provided to you by GPLv2 license."""


import os
import sys
import argparse


class MDP(object):
    """
    class MDP are designed to generate kinds of mdp files

    """

    def __init__(self) -> None:
        """init the MDP class"""

        self.application_loc = {
            "ions": os.path.join("data", "ions.mdp"),
            "em": os.path.join("data", "em.mdp"),
            "nvt": os.path.join("data", "nvt.mdp"),
            "npt": os.path.join("data", "npt.mdp"),
            "md": os.path.join("data", "md.mdp"),
            "blank": os.path.join("data", "blank.mdp"),
        }

        print("Info -> MDP module could init some mdp templates for you.\n", end="")
        print("        Specify application (-a) to generate a mdp file. ")
        print("Info -> applications to choose: ions, em, nvt, npt, md, blank")
        print("\nWARNING -> the generated mdp file may be not appropriate", end="")
        print(" for your system, CHECK IT YOURSELF !\n")

    def gen_mdp(self, outmdp: str, application: str) -> None:
        """gen mdp template by specified application"""

        ## check parameters
        if outmdp == None or len(outmdp) <= 4 or outmdp[-4:] != ".mdp":
            print("Error -> please specify an output file name with suffix .mdp")
            exit()
        if os.path.exists(outmdp):
            print("Error -> {} is already in current directory".format(outmdp))
            exit()
        if application == None or application not in self.application_loc.keys():
            print(
                "Info -> application available:\n         {}".format(
                    " ".join(self.application_loc.keys())
                )
            )
            print("Error -> no application {} found".format(application))
            exit()

        ## gen mdp
        data_file_path = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__))
        )
        with open(
            os.path.join(data_file_path, self.application_loc[application]), "r"
        ) as fo:
            content = fo.read()
        with open(outmdp, "w") as fo:
            fo.write(content)

        print(
            "Info -> generate {}.mdp for {} application successfully".format(
                outmdp, application
            )
        )


def mdp_gen(outmdp: str, application: str) -> None:
    """gen mdp templates"""

    mdp = MDP()
    mdp.gen_mdp(outmdp, application)


def mdp_call_functions(arguments: list = None):
    """call functions according to arguments"""

    if arguments == None:
        arguments = [argv for argv in sys.argv]

    ## parse the command parameters
    parser = argparse.ArgumentParser(description="generate mdp file templates")
    parser.add_argument("-o", "--output", help="file name to output")
    parser.add_argument(
        "-a",
        "--application",
        choices=["ions", "em", "nvt", "npt", "md", "blank"],
        help="specify the application of mdp, choices: ions, em, nvt, npt, md, blank",
    )

    if len(arguments) < 2:
        print("Error -> no input parameters, -h or --help for help messages")
        exit()
    method = arguments[1]
    # print(method)
    if method in ["-h", "--help"]:
        parser.parse_args(arguments[1:])
        exit()
    args = parser.parse_args(arguments[2:])
    if method == "mdp_gen":
        mdp_gen(args.output, args.application)
    else:
        print("Error -> unknown method {}".format(method))
        exit()

    print("Info -> good day !")


def main():
    mdp_call_functions()


if __name__ == "__main__":
    main()
